﻿namespace ProjectCleaningCar.CollectionGenericObjects;
using Exceptions;
using ProjectCleaningCar.Drawning;

public class ListGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
	/// Список объектов, которые храним
	/// </summary>
	private readonly List<T?> _collection;

    /// <summary>
    /// Максимально допустимое число объектов в списке
    /// </summary>
    public int _maxCount;

    public int Count => _collection.Count;

    public int MaxCount { 
        get
        {
            return _collection.Count;
        }
        set 
        { 
            if (value > 0) 
            { 
                _maxCount = value; 
            } 
        } 
    }

    public CollectionType GetCollectionType => CollectionType.List;
    /// <summary>
	/// Конструктор
	/// </summary>
	public ListGenericObjects()
    {
        _collection = new();
    }

    public T Get(int position)
    {
        if (position >= Count || position < 0) throw new PositionOutOfCollectionException();
        return _collection[position];
    }

    public int Insert(T obj, IEqualityComparer<T?>? comparer = null)
    {
        if (Count == _maxCount) throw new CollectionOverflowException();
        if (comparer != null)
        {
            if (_collection.Contains(obj, comparer))
            {
                throw new ObjectAlreadyExistsException(obj);
            }
        }
        _collection.Add(obj);
        return Count;
    }
    public int Insert(T obj, int position, IEqualityComparer<T?>? comparer = null)
    { 
        if (Count == _maxCount) throw new CollectionOverflowException();
        if (position >= Count || position < 0) throw new PositionOutOfCollectionException();
        if (comparer != null)
        {
            if (_collection.Contains(obj, comparer))
            {
                throw new ObjectAlreadyExistsException(obj);
            }
        }
        _collection.Insert(position, obj);
        return position;
    }

    public T Remove(int position)
    {
        if (position >= Count || position < 0) throw new PositionOutOfCollectionException();
        T temp = _collection[position];
        _collection.RemoveAt(position);
        return temp;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Count; ++i)
        {
            yield return _collection[i];
        }
    }

    public void CollectionSort(IComparer<T?> comparer)
    {
        _collection.Sort(comparer);
    }
}
